__title__ = "Japanera"
__description__ = "Easy japanese era tool"
__url__ = "https://github.com/nagataaaas/Japanera"
__version_info__ = ("2", "1", "1")
__version__ = ".".join(__version_info__)
__author__ = "Yamato Nagata"
__author_email__ = "nagata@nagata.pro"
__maintainer__ = "Yamato Nagata"
__license__ = "MIT"
__copyright__ = "(c) 2019-2024 by Yamato Nagata"
